# 文件系统

实现：沈博洋

初赛系统调用完成进度：16 / 16



## 总览

<img src="img/Figure-1.1.png" alt="image1" style="zoom: 15%;" />

在 linux “一切皆文件”的设计思想下，无论是文件、管道还是设备，都可以用统一的文件描述符来形容，同时具有读写接口。为了达到这一目的，需要分别实现后再用统一的接口层为应用程序提供服务。

最顶层是系统调用层，为系统调用提供具体的实现。第二层是文件描述层，这层主要是提供不同实现的统一描述，并在系统调用时进行不同部分的分发。第三层在初赛分为了3个部分，最左侧是FAT文件系统部分，中间是管道部分，最右侧是硬件部分。

FAT文件系统总共分为6层，从底向上，第一层是 `virtio` 驱动层，用于与磁盘沟通；第二层是块缓存层，将读取到的磁盘利用LRU策略缓存起来，避免了频繁的磁盘沟通。第三层是fat层，提供了对磁盘FAT文件系统的基本读写功能。第四层是簇缓存层，因为FAT组织簇时呈现链表形式，对于非顺序的读写（比如对于某个目录项）支持并不好，此层提供了对于簇号的缓存。第五层是目录元数据层，可以看做FAT文件系统在内存中的抽象。第六层是目录缓冲层，避免了对目录操作时频繁的写回。

对于管道的实现，是在内存中划分一块贡献内存，并从文件的角度进行描述。对于设备，初赛只涉及串口驱动，将读写函数与文件的读写功能绑定即可。

鉴于SD卡为FAT32文件系统，GalaxyOS将xv6原本的文件系统替换成了FAT32文件系统，并且尽可能地与原文件系统的接口保持一致或类似。目前初赛阶段在qemu平台上可以运行用户程序，包括shell。



## 相较于原xv6修改了什么

对于文件系统，GalaxyOS保留了xv6的disk、buf和file descriptor这几层的实现，并做出了以下修改：

- FAT32不支持日志系统，GalaxyOS去除了xv6文件系统的log层 `log.c`；
- FAT32没有 `inode` ，文件的元数据直接存放在目录项中，因此GalaxyOS将原本文件结构体 `inode`替换为目录项结构体 `dirent` (directory entry)；
- 重新实现xv6文件系统 `fs.c` 中的各个函数，将函数接口中的 `inode` 替换成了 `entry` ，函数命名上保留原函数的特征但也做了修改以示区分，如`ilock`变为`elock`、`writei`变为`ewrite`等等；
- 关于buf层，由于FAT32的一个簇的大小较大，并且依不同的存储设备而变，因此GalaxyOS目前以扇区为单位作缓存。



## 系统调用

### 1. getcwd

通过递归的方式来获取路径名：

- 首先检查给定的 `dirent` 是否是根目录（即 `/`）。如果是根目录，直接返回 `/`。
- 否则，它会递归地获取父目录的路径名，并将当前目录的名称附加到父目录的路径名后面。

最后，返回对应地址。

### 2. pipe2

即创建管道的操作。写入管道写入端的数据由内核缓冲，直到从管道的读取端读取。创建管道失败或者传输数据的过程失败需要进行相应的错误处理。

### 3. dup

将文件描述符对应的文件结构体中的引用次数自增1后，返回新的复制的文件描述符即可。

### 4. dup3

相较于 `dup` ，需要将新的文件描述符覆盖老的文件描述符的内容，然后将引用次数自增1，返回新的文件描述符；如果 `oldfd` 与 `newfd` 的值相同，则直接返回 `newfd` 即可。

### 5. chdir

将传入的路径名判断是否为文件夹类型后进行一次查找操作，找到对应的 `dirent` 后将当前 `proc` 的 `cwd` (当前工作目录) 修改为该 `dirent` 即可。

### 6. openat

需要对于不同的 `flags` 组合做出不同的情况讨论，包括相对路径的处理等等，初赛阶段对于一些可能出错的情况只是做了一些简单的恢复现场后返回的粗糙处理。最后统一的步骤为将处理好的文件结构体 `f` 的 `type`, `off`, `ep`, `readable`, `writable`字段进行对应修改。

### 7. close

首先将该文件描述符的类型修改为 `FD_NONE` ，并将引用次数修改为0，然后需要对不同的文件类型执行不同类型的关闭操作：若为目录则执行FAT32的 `eput` 操作，若为管道则执行关闭管道的操作。由于FAT32的文件属性不支持设备类型，因此不对也无法对设备类型的文件执行关闭操作。

### 8. getdents64

首先遍历`dirent`，每次偏移为32。调用`enext`读取目录，如果返回值为0则代表`entry`为空，跳过；如果返回值为-1则代表目录结束，返回；如果返回值为1，则代表找到一个文件，于是填充`linux_dirent64`结构体的属性并将结构体复制到buf区。

### 9. read

与 `close` 通过文件类型区分关闭操作类似， `read` 也需要对文件类型进行判断：若为目录则执行FAT32的 `eread` 操作并加上对应的文件偏移量，若为管道则执行读管道 `piperead` 的操作，若为设备则执行读设备的操作。最后返回每个操作中读取的字节数即可。

### 10. write

 `write` 与 `read` 操作同理，只是将读操作变为了写操作：若为目录则执行FAT32的 `ewrite` 操作并加上对应的文件偏移量，若为管道则执行写管道 `pipewrite` 的操作，若为设备则执行写设备的操作。最后返回每个操作中写入的字节数即可。

### 11. linkat

新建一个文件，类型为链接，内容是链接的绝对路径。在 `lookup_path` 的时候，如果是链接文件，则跳转该文件记录的真实文件。

### 12. unlinkat

首先对目录和文件类型进行判断，然后进行相对目录的判断。初赛阶段使用了FAT32中的 `eremove` 操作来达到删除链接的操作。

### 13. mkdirat

与 `mkdir` 类似，在读入参数后进行文件类型与相对路径的判断，然后在指定目录位置调用FAT32的 `ealloc` 操作即可。

### 14. umount2

首先通过FAT32的操作 `ename` 找到指定卸载目录的文件描述符，然后调用FAT32的 `umonut2` 操作，即将`mount_flag` 修改为0，同时将 `mnt` 对应的 `devs` 的区域的内容清空为0即可。

### 15. mount

将参数传入后进行一次设备类型的判断，然后调用FAT32的 `mount` 操作，需要对应修改 `mnt` 对应的 `devs` 的区域的内容。

### 16. fstat

将传入的路径名判断是否为文件夹类型后，调用一次FAT32的 `ekstat` 操作，然后将获取的指针复制到接收保存文件状态的指针位置后即可。



## 遇到的问题

对FAT32的支持只是一个简单的实现，还未经过严格的测试，存在许多不足：

- 为了简单起见，目前新建文件时无法生成时间戳，将长文件名转换到短文件名时未排重（这会影响较低版本的 FAT 系统）；
- FAT32的文件属性不支持设备类型，而原xv6文件系统中有`mknod`系统调用新建一个设备，并可以使用`open`打开，而控制台输入输出就是这样打开的。为了在修改后的文件系统打开控制台标准输入输出，GalaxyOS改用了另一种实现，即直接在内存中初始化控制台。

同时，在系统调用的实现过程中，遇到了包括但不限于以下的问题：

- 原xv6系统中的 `param.h` 文件中的 `NOFILE` 的值设置的过小，导致 `dup3` 将过大的文件描述符识别为无效的文件描述符。
- `argfd` 的实现在适应FAT32系统时出现了一些问题，初赛阶段未予以解决，而使用其他方式读取绕过了这一问题。
