/*
  Memory Management
  Feature: segment struct, used to record segments in vm.
*/

#include "include/types.h"
#include "include/riscv.h"
#include "include/param.h"
#include "include/syscall.h"
#include "include/defs.h"
#include "include/mm.h"
#include "include/proc.h"

// Create segment node
int
new_seg(enum seg_type type, uint64 start, uint64 sz, int flag)
{
  #ifdef DEBUG
    //printf("creating new segment.\n");
  #endif
  struct proc *p = myproc();
  int i, j, found = 0;

  for(i = 0; i < NSEG; i++) {
    if(!found && p->segments[i].valid == 0) {
      j = i;
      found = 1;
      #ifdef DEBUG
        //printf("found available segment.\n");
      #endif
      if (start == 0)
        break;
    }
    else if (start != 0 && (start + sz >= p->segments[i].addr) && (start <= p->segments[i].addr + p->segments[i].sz)) {
      printf("Segment overlapping!\n");
      return -1;
    }
  }

  if(found == 1) {
    p->segments[j].valid = 1;
    p->segments[j].type = type;
    if (start)
      p->segments[j].addr = start;
    else
      p->segments[j].addr = p->sz;
    p->segments[j].sz = sz;
    p->segments[j].mmap = NULL;
    p->segments[j].flag = flag;
    
    #ifdef DEBUG
        //printf("returning with %d.\n", j);
    #endif
    return j;
  }
  else
    return -1;
}

int
locate_seg(enum seg_type type, uint64 start, uint sz)
{
  struct proc *p = myproc();

  int i;
  for(i = 0; i < NSEG; i++) {
    if(p->segments[i].valid == 1 && p->segments[i].type == type && p->segments[i].addr == start && p->segments[i].sz == sz)
      return i;
  }

  return -1;
}