//
// Memory-management system calls.
//

#include "include/types.h"
#include "include/riscv.h"
#include "include/param.h"
#include "include/proc.h"
#include "include/syscall.h"
#include "include/mm.h"
#include "include/defs.h"

uint64
sys_brk(void)
{
  uint64 new_addr, old_addr;
  int n;

  argaddr(0, &new_addr);
  old_addr = myproc()->sz;

  if(new_addr == 0)
    return old_addr;

  n = new_addr - old_addr;

  if(growproc(n) < 0)
    return -1;
  return new_addr;
}

uint64
sys_munmap(void)
{
  uint64 start, len;

  argaddr(0, &start);
  argaddr(1, &len);

  return do_munmap(start, len);
}

uint64
sys_mmap(void)
{
  uint64 start, len, off;
  int prot, flags, fd;

  argaddr(0, &start);
  argaddr(1, &len);
  argint(2, &prot);
  argint(3, &flags);
  argint(4, &fd);
  argaddr(5, &off);

  // Filter invalid args before doing mmap.

  if (off < 0 || off % PGSIZE != 0 || len <= 0)
    return -1;

  if (!(flags & (MAP_SHARED | MAP_PRIVATE)))
    return -1;

  if (!start && (flags & MAP_FIXED))
    return -1;

  if (fd < 0 || fd > NOFILE)
    return -1;

  if (!fd && (flags & MAP_ANONYMOUS))
    return -1;

  return do_mmap(start, len, prot, flags, fd, off);
}