#include "include/types.h"
#include "include/riscv.h"
#include "include/param.h"
#include "include/proc.h"
#include "include/syscall.h"
#include "include/times.h"
#include "include/vm.h"
#include "include/uname.h"
#include "include/string.h"
#include "include/timer.h"

const char SYSNAME[] = "galaxyos";
const char NODENAME[] = "none";
const char RELEASE[] = "none";
const char VERSION[] = "early-alpha";
const char MACHINE[] = "visionfive-2";
const char DOMAINNAME[] = "none";

uint64
sys_times(void)
{
  uint64 tms;
  argaddr(0, &tms);

  struct proc *p = myproc();

  if (tms) {
    if(copyout2(tms, (char*)&p->proc_tms, sizeof(p->proc_tms)) == -1)
      return -1;
  }
  else
    return -1;

  return r_time();
}

uint64
sys_uname(void)
{
  uint64 addr;
  argaddr(0, &addr);

  struct uname* uts = (struct uname*)addr;

  if (copyout2((uint64)uts->sysname, (void*)SYSNAME, sizeof(SYSNAME)) < 0) 
    return -1;
	if (copyout2((uint64)uts->nodename, (void*)NODENAME, sizeof(NODENAME)) < 0) 
    return -1;
	if (copyout2((uint64)uts->release, (void*)RELEASE, sizeof(RELEASE)) < 0)
    return -1;
	if (copyout2((uint64)uts->version, (void*)VERSION, sizeof(VERSION)) < 0) 
		return -1;
	if (copyout2((uint64)uts->machine, (void*)MACHINE, sizeof(MACHINE)) < 0) 
		return -1;
	if (copyout2((uint64)uts->domainname, (void*)DOMAINNAME, sizeof(DOMAINNAME)) < 0) 
		return -1;

  return 0;
}

uint64
sys_sched_yield(void)
{
  yield();
  return 0;
}

uint64
sys_gettimeofday(void)
{
  uint64 addr;
  argaddr(0, &addr);

  struct timeval ts;
  uint64 timestamp = r_time();
  ts.tv_sec = timestamp / CLK_FRQ;
  ts.tv_usec = timestamp / (CLK_FRQ / 100000);
  ts.tv_usec = (ts.tv_usec % 100000) * 10;

  if(addr) {
    if (copyout2(addr, (char*)&ts, sizeof(ts)))
      return -1;
  }
  else
    return -1;

  return 0;
}

uint64
sys_nanosleep(void)
{
  uint64 req_addr, rem_addr;
  argaddr(0, &req_addr);
  argaddr(1, &rem_addr);

  struct timeval req_time, rem_time;
  if (copyin2((char*)&req_time, req_addr, sizeof(req_time)) < 0)
    return -1;
  if (copyin2((char*)&rem_time, rem_addr, sizeof(rem_time)) < 0)
    return -1;

  uint64 n = req_time.tv_sec;

  acquire(&tickslock);
  uint ticks0 = ticks;
  while(ticks - ticks0 < n){
    if(myproc()->killed){
      release(&tickslock);
      return -1;
    }
    sleep(&ticks, &tickslock);
  }
  release(&tickslock);

  return 0;
}